package view;


import java.awt.Color;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;

public class DropButton extends JButton {
  
  public DropButton (DoubleString ds) {    
    this.setText(ds.getText());
    this.setName(ds.getName());
    this.borderColor=Color.green;
    this.dtListener = new DTListener();
    
    // component, ops, listener, accepting
    this.dropTarget = new DropTarget(this, 
      this.acceptableActions,
      this.dtListener,
      true);
  }
  
  public DropButton (DoubleString ds, int a) {
    if (a != DnDConstants.ACTION_NONE &&
      a != DnDConstants.ACTION_COPY &&
      a != DnDConstants.ACTION_MOVE &&
      a != DnDConstants.ACTION_COPY_OR_MOVE &&      
      a != DnDConstants.ACTION_LINK
    ) throw new IllegalArgumentException("action" + a);
    
    this.acceptableActions = a;
    this.setText(ds.getText());
    this.setName(ds.getName());
    this.borderColor=Color.green;
    this.dtListener = new DTListener();
    
    // component, ops, listener, accepting
    this.dropTarget = new DropTarget(this, 
      this.acceptableActions,
      this.dtListener,
      true);
    
  }

  private void showBorder(boolean b) {
    if(b) {
      this.setBorder( BorderFactory.createLineBorder(this.borderColor, 10) );      
    } else {
      this.setBorder( BorderFactory.createEmptyBorder() );      
    }
    this.getParent().validate();    
    this.repaint();    
  }
  
  /**
   * DTListener
   * a listener that tracks the state of the operation
   * @see java.awt.dnd.DropTargetListener
   * @see java.awt.dnd.DropTarget
   */
  class DTListener implements DropTargetListener {
    
    /**
     * Called by isDragOk
     * Checks to see if the flavor drag flavor is acceptable
     * @param e the DropTargetDragEvent object
     * @return whether the flavor is acceptable
     */
    private boolean isDragFlavorSupported(DropTargetDragEvent e) {
      boolean ok=false;
      if (e.isDataFlavorSupported(StringTransferable.plainTextFlavor)) {
	ok=true;
      } else if (e.isDataFlavorSupported(
	StringTransferable.localStringFlavor)) {
	ok=true;
      } else if (e.isDataFlavorSupported(DataFlavor.stringFlavor)) {	  
	ok=true;
      } else if (e.isDataFlavorSupported(DataFlavor.plainTextFlavor)) {
	ok=true;
      }
      return ok;
    }
    /**
     * Called by drop
     * Checks the flavors and operations
     * @param e the DropTargetDropEvent object
     * @return the chosen DataFlavor or null if none match
     */
    private DataFlavor chooseDropFlavor(DropTargetDropEvent e) {
      if (e.isLocalTransfer() == true &&
	//e.isDataFlavorSupported(StringTransferable.localStringFlavor)) {
    		  e.isDataFlavorSupported(StringTransferable.localDoubleStringFlavor)) {
	return StringTransferable.localDoubleStringFlavor;
      }
      DataFlavor chosen = null;      
      if (e.isDataFlavorSupported(StringTransferable.doubleStringFlavor)) {	  
	chosen = StringTransferable.doubleStringFlavor;
      } else if (e.isDataFlavorSupported(StringTransferable.localDoubleStringFlavor)) {	  
    		chosen = StringTransferable.doubleStringFlavor;
      } else if (e.isDataFlavorSupported(StringTransferable.localStringFlavor)) {
	chosen = StringTransferable.localStringFlavor;	
      } else if (e.isDataFlavorSupported(DataFlavor.stringFlavor)) {	  
	chosen = DataFlavor.stringFlavor;
      } else if (e.isDataFlavorSupported(DataFlavor.plainTextFlavor)) {
	chosen = DataFlavor.plainTextFlavor;	
      } 
      return chosen;
    }

    /**
     * Called by dragEnter and dragOver
     * Checks the flavors and operations
     * @param e the event object
     * @return whether the flavor and operation is ok
     */
    private boolean isDragOk(DropTargetDragEvent e) {
      if(isDragFlavorSupported(e) == false) {
	return false;
      }
      
      // the docs on DropTargetDragEvent rejectDrag says that
      // the dropAction should be examined
      int da = e.getDropAction();      
      
      // we're saying that these actions are necessary      
      if ((da & DropButton.this.acceptableActions) == 0)
	return false;
      return true;
    }

    /**
     * start "drag under" feedback on component
     * invoke acceptDrag or rejectDrag based on isDragOk
     */
    public void dragEnter(DropTargetDragEvent e) {    
      if(isDragOk(e) == false) {
	DropButton.this.borderColor=Color.red;            
	showBorder(true);
	e.rejectDrag();      
	return;
      }
      DropButton.this.borderColor=Color.green;      
      showBorder(true);
      e.acceptDrag(e.getDropAction());      
    }

    /**
     * continue "drag under" feedback on component
     * invoke acceptDrag or rejectDrag based on isDragOk
     */
    public void dragOver(DropTargetDragEvent e) {
      if(isDragOk(e) == false) {
	DropButton.this.borderColor=Color.red;            
	showBorder(true);
	e.rejectDrag();      
	return;
      }
      e.acceptDrag(e.getDropAction());            
    }
    
    public void dropActionChanged(DropTargetDragEvent e) {
      if(isDragOk(e) == false) {
	e.rejectDrag();      
	return;
      }
      e.acceptDrag(e.getDropAction());            
    }
    
    public void dragExit(DropTargetEvent e) {
      DropButton.this.borderColor=Color.green;            
      showBorder(false);
    }

    /**
     * perform action from getSourceActions on
     * the transferrable
     * invoke acceptDrop or rejectDrop
     * invoke dropComplete
     * if its a local (same JVM) transfer, use StringTransferable.localStringFlavor
     * find a match for the flavor
     * check the operation
     * get the transferable according to the chosen flavor
     * do the transfer
     */
    public void drop(DropTargetDropEvent e) {
      
      DataFlavor chosen = chooseDropFlavor(e);
      if (chosen == null) {
	e.rejectDrop();      	
	return;
      }

      // the actual operation
      int da = e.getDropAction();
      // the actions that the source has specified with DragGestureRecognizer
      int sa = e.getSourceActions();            
      if ( ( sa & DropButton.this.acceptableActions ) == 0 ) {
	e.rejectDrop();      		
	showBorder(false);      		
	return;
      }
    
      Object data=null;
      try {
	/*
	 * the source listener receives this action in dragDropEnd.
	 * if the action is DnDConstants.ACTION_COPY_OR_MOVE then
	 * the source receives MOVE!
	 */
	e.acceptDrop(DropButton.this.acceptableActions); 
  
	data = e.getTransferable().getTransferData(chosen);
	if (data == null)
	  throw new NullPointerException();
      } catch ( Throwable t ) {
	t.printStackTrace();
	e.dropComplete(false);
	showBorder(false);      		
	return;
      }
      
      if (data instanceof String ) {
	String s = (String) data;
	DropButton.this.setText(s);	
      } else if (data instanceof DoubleString) {
    	  DoubleString ds = (DoubleString) data;
    	  DropButton.this.setText(ds.getText());
    	  DropButton.this.setName(ds.getName());
      } else if (data instanceof InputStream) {
	InputStream input = (InputStream)data;
	InputStreamReader isr = null;
	//	BufferedReader br = null;
	try {
	  //	  br = new BufferedReader(isr=new InputStreamReader(input,"Unicode"));
	  isr=new InputStreamReader(input,"Unicode");	  
	} catch(UnsupportedEncodingException uee) {
	  isr=new InputStreamReader(input);	  	  
	}

	StringBuffer str = new StringBuffer();
	int in=-1;
	try {
	  while((in = isr.read()) >= 0 ) {
	    if (in != 0)
	      str.append((char)in);
	  }

	  DropButton.this.setText(str.toString());
	  
	} catch(IOException ioe) {
		
	  e.dropComplete(false);      	  
	  showBorder(false);
	  String message = "Bad drop\n" + ioe.getMessage();
	  JOptionPane.showMessageDialog(DropButton.this,
	    message,
	    "Error",
	    JOptionPane.ERROR_MESSAGE);
	  return;
	}

      } else {
	e.dropComplete(false);      	  	
	showBorder(false);      	
	return;
      }
      
      e.dropComplete(true);      
      showBorder(false);      	      
    }

  }  
  
  private DropTarget dropTarget;
  private DropTargetListener dtListener;

  /**
   * the actions supported by this drop target
   */
  
  private int acceptableActions = DnDConstants.ACTION_COPY;  
  
  private Color borderColor;
}
